import * as line from "@line/bot-sdk";
import DBTools from "./DBTools";

/**
 * GroupBuy
 */
export default class GroupBuy {

	//#region Custom

	public static async Run(event: any, msg: string[], bot: line.Client): Promise<line.Message | line.Message[]> {
		let message: line.Message | line.Message[] = null;
		const instruction: string = msg[1];
		switch (instruction) {
			case "開團": {
				message = await this.startBuying(event, msg, bot);
				break;
			}

			case "購買": {
				message = await this.buying(event, msg, bot);
				break;
			}

			case "查詢": {
				message = await this.searchBuying(event, msg[2], bot);
				break;
			}

			case "收單": {
				message = await this.stopBuying(event, msg[2], bot);
				break;
			}

			default:
				break;
		}
		return message;
	}

	/**
	 * 開始團購
	 * @param msg
	 */
	private static async startBuying(event: any, msg: any[], bot: line.Client): Promise<line.Message | line.Message[]> {
		const storename: string = msg[2];
		const query: string = String.Format("INSERT INTO `line_catan_groupbuy`.`groupbuydata` (`storename`) VALUES ('{0}');"
			, storename);
		const queryresp: any = await DBTools.Query(query);

		const message: line.Message = {
			type: "text",
			text: storename + " 已開團成功"
		};
		return message;
	}

	/**
	 * 購物
	 * @param msg
	 */
	private static async buying(event: any, msg: any[], bot: line.Client): Promise<line.Message | line.Message[]> {
		const storename: string = msg[2];
		const food: string = msg[3];
		const count: number = +msg[4];
		let note: string = msg[5] ?? "";
		const userId: string = event.source.userId;
		const userdata: line.Profile = await bot.getGroupMemberProfile(event.source.groupId, userId);

		const query: string = String.Format("SELECT * FROM `line_catan_groupbuy`.`groupbuydata` WHERE `storename` = '{0}' ORDER BY `storename` DESC LIMIT 1;", storename);
		const queryresp: any[] = await DBTools.Query(query);
		if (queryresp.length === 0) {
			const message: line.Message = {
				type: "text",
				text: `錯誤`
			};
			return message;
		}
		const querydata: any = queryresp[0];
		if (!querydata.isopen) {
			const message: line.Message = {
				type: "text",
				text: `團購未開放`
			};
			return message;
		}

		let totalCount: number = count;
		let data: any = querydata.data ? JSON.parse(querydata.data) : {};
		if (data[food]) {
			const userbuydata: any = data[food][userId];
			if (userbuydata) {
				totalCount = userbuydata.count = count;
				if (note) {
					userbuydata.note = note;
				} else {
					note = userbuydata.note;
				}
			} else {
				const userbuydata: any = {
					count: count,
					note: note
				};
				data[food][userId] = userbuydata;
			}
		} else {
			const userbuydata: any = {
				count: count,
				note: note
			};
			const adddata: Object = {};
			adddata[userId] = userbuydata;
			data[food] = adddata;
		}
		const data_str: string = JSON.stringify(data);
		const queryAddData: string = String.Format("UPDATE `line_catan_groupbuy`.`groupbuydata` SET `data` = '{0}' WHERE `id` = {1};"
			, data_str, querydata.id);
		await DBTools.Query(queryAddData);
		let test: string = `${userdata.displayName} 已${count > 0 ? "增加" : "減少"}成功 ${querydata.storename}的${food} ${count}個, 總共${totalCount}個`;
		if (note) {
			test += ` 備註: ${note}`;
		}
		const message: line.Message = {
			type: "text",
			text: test
		};
		return message;
	}

	private static async searchBuying(event: any, storename: string, bot: line.Client): Promise<line.Message | line.Message[]> {
		let resp: line.Message | line.Message[] = {
			type: "text",
			text: "沒有資料"
		};
		const query: string = String.Format("SELECT * FROM `line_catan_groupbuy`.`groupbuydata` WHERE `storename` = '{0}' ORDER BY `id` DESC LIMIT 1;", storename);
		const queryresp: any[] = await DBTools.Query(query);
		if (queryresp && queryresp.length > 0) {
			resp = await this.getData_LineFlex(event, queryresp[0], bot);
		} else {
			const message: line.Message = {
				type: "text",
				text: `沒有符合的選項`
			};
			return message;
		}
		return resp;
	}

	private static async stopBuying(event: any, storename: string, bot: line.Client): Promise<line.Message | line.Message[]> {
		let resp: line.Message | line.Message[] = {
			type: "text",
			text: "沒有資料"
		};
		const query: string = String.Format("SELECT * FROM `line_catan_groupbuy`.`groupbuydata` WHERE `storename` = '{0}' AND `isopen` = '1' ORDER BY `id` DESC LIMIT 1;", storename);
		const queryresp: any[] = await DBTools.Query(query);
		if (queryresp.length > 0) {
			const query1: string = String.Format("UPDATE `line_catan_groupbuy`.`groupbuydata` SET `isopen` = 0 WHERE `id` = {0};", queryresp[0].id);
			const queryresp1: any[] = await DBTools.Query(query1);
			resp = await this.searchBuying(event, storename, bot);
		} else {
			const message: line.Message = {
				type: "text",
				text: `沒有符合的選項`
			};
			return message;
		}
		return resp;
	}

	/**
	 * 取得名單Line模板
	 * @param storename
	 * @param matchs
	 */
	private static async getData_LineFlex(event: any, queryresp: any, bot: line.Client): Promise<line.TextMessage | line.Message[]> {
		let data_LineFlex: string = queryresp.storename;
		const data_Str: string = queryresp.data;
		const names: string[] = [];
		if (data_Str) {
			const data: any = JSON.parse(data_Str);
			for (let i: number = 0, data_key: string[] = Object.keys(data); i < data_key.length; i++) {
				const food: string = data_key[i];
				const shopping: Object = data ? data[food] : {};
				data_LineFlex += `\n\n品項: ${food}`;
				for (let j: number = 0, shopping_key: string[] = Object.keys(shopping); j < shopping_key.length; j++) {
					const userId: string = shopping_key[j];
					const userbuydata: any = shopping[userId];
					const count: number = userbuydata.count;
					const note: string = userbuydata.note;
					if (count) {
						let name: string = names[userId];
						if (!name) {
							const userdata: line.Profile = await bot.getGroupMemberProfile(event.source.groupId, userId);
							name = names[userId] = userdata?.displayName ?? userId;
						}

						data_LineFlex += `\n${name} ${count}個` + (note ? `, 備註: ${note}` : "");
					}
				}
			}
		} else {
			data_LineFlex += `\nㄏ  沒人買`;
		}
		const messages: line.Message = {
			type: "text",
			text: data_LineFlex
		};
		return messages;
	}

	//#endregion
}